@@ -6,19 +6,152 @@ API: Additional recipes
 
 # Built-ins
 from __future__ import absolute_import, division, print_function
-from itertools import chain, islice, repeat
+from collections import OrderedDict
+from itertools import chain, islice, repeat, product, combinations
+from operator import itemgetter
 
 # This module
-from .. import PY2, nth, unique_justseen
+from .. import EQ_PY2, nth, unique_justseen, chained
 from ._core import tail
 
 
-if PY2:
+if EQ_PY2:
     from itertools import imap as map
 
 
-__all__ = ['getitem', 'insert', 'itersubclasses', 'pad', 'remove', 'replace',
-           'replicate']
+__all__ = ['argsorted', 'combinations_from_relations', 'getitem',
+           'insert', 'itersubclasses', 'pad', 'remove', 'replace', 'replicate']
+
+
+def argsorted(iterable, key=None, reverse=False):
+    """Returns the indices that would sort the `iterable`.
+
+    Parameters
+    ----------
+    iterable : iterable
+        The `iterable` to sort.
+
+    key : callable, None, optional
+        If ``None`` sort the items in the `iterable`, otherwise sort the
+        ``key(items)``.
+        Default is ``None``.
+
+    reverse : bool, optional
+        If ``False`` sort the `iterable` in increasing order otherwise in
+        decreasing order.
+        Default is ``False``.
+
+    Returns
+    -------
+    sortindices : list
+        The indices that would sort the `iterable`.
+
+    Notes
+    -----
+    See :py:func:`sorted` for more explanations to the parameters.
+
+    Examples
+    --------
+    To get the indices that would sort a sequence in increasing order::
+
+        >>> from iteration_utilities import argsorted
+        >>> argsorted([3, 1, 2])
+        [1, 2, 0]
+
+    It also works when sorting in decreasing order::
+
+        >>> argsorted([3, 1, 2], reverse=True)
+        [0, 2, 1]
+
+    And when applying a `key` function::
+
+        >>> argsorted([3, 1, -2], key=abs)
+        [1, 2, 0]
+    """
+    if key is None:
+        key = itemgetter(1)
+    else:
+        key = chained(itemgetter(1), key)
+    return [i[0] for i in sorted(enumerate(iterable),
+                                 key=key, reverse=reverse)]
+
+
+def combinations_from_relations(dictionary, r):
+    """Yield combinations where only one item (or None) of each equivalence
+    class is present.
+
+    Parameters
+    ----------
+    dictionary : dict with iterable values or convertible to one.
+        A dictionary defining the equivalence classes, each key should contain
+        all equivalent items as it's value.
+
+        .. warning::
+            Each ``value`` in the `dictionary` must be iterable.
+
+        .. note::
+            If the dictionary isn't ordered then the order in the
+            combinations and their order of appearance is not-deterministic.
+
+        .. note::
+            If the `dictionary` isn't dict-like it will be converted to an
+            OrderedDict.
+
+    r : int or None, optional
+        The number of combinations, if ``None`` it defaults to the length of
+        the `dictionary`.
+
+    Returns
+    -------
+    combinations : generator
+        The combinations from the dictionary.
+
+    Examples
+    --------
+    In general the :py:class:`collections.OrderedDict` should be used to call
+    the function. But it will also be automatically converted to one if
+    one inserts an iterable that is convertible to a dict::
+
+        >>> from iteration_utilities import combinations_from_relations
+
+        >>> classes = [('a', [1, 2]), ('b', [3, 4]), ('c', [5, 6])]
+        >>> for comb in combinations_from_relations(classes, 2):
+        ...     print(comb)
+        (1, 3)
+        (1, 4)
+        (2, 3)
+        (2, 4)
+        (1, 5)
+        (1, 6)
+        (2, 5)
+        (2, 6)
+        (3, 5)
+        (3, 6)
+        (4, 5)
+        (4, 6)
+
+    This is equivalent to creating the :py:class:`collections.OrderedDict`
+    manually::
+
+        >>> from collections import OrderedDict
+        >>> odct = OrderedDict(classes)
+        >>> for comb in combinations_from_relations(odct, 3):
+        ...     print(comb)
+        (1, 3, 5)
+        (1, 3, 6)
+        (1, 4, 5)
+        (1, 4, 6)
+        (2, 3, 5)
+        (2, 3, 6)
+        (2, 4, 5)
+        (2, 4, 6)
+    """
+    if not isinstance(dictionary, dict):
+        dictionary = OrderedDict(dictionary)
+
+    for keycomb in combinations(dictionary, r):
+        for valuecomb in product(*itemgetter(*keycomb)(dictionary)):
+            yield valuecomb
 
 
 def itersubclasses(cls, seen=None):
@@ -120,7 +253,7 @@ def pad(iterable, fillvalue=None, nlead=0, ntail=0):
 
     nlead, ntail : int or None, optional
         The number of times to pad in front (`nlead`) and after (`ntail`) the
-        `iterable. If `ntail` is ``None`` pad indefinitly (not possible for
+        `iterable`. If `ntail` is ``None`` pad indefinitly (not possible for
         `nlead`).
         Default is ``0``.
 
@@ -145,9 +278,8 @@ def pad(iterable, fillvalue=None, nlead=0, ntail=0):
     [1, 2, 3, 0, 0, 0, 0, 0, 0, 0]
 
     .. warning::
-        This will return an infinitly long generator if either ``before`` or
-        ``after`` are ``None``, so do not try to do something like
-        ``list(pad([], before=None))``!
+        This will return an infinitly long generator if ``ntail`` is ``None``,
+        so do not try to do something like ``list(pad([], ntail=None))``!
     """
     prepend = repeat(fillvalue, nlead)
 
