@@ -20,7 +20,7 @@ from itertools import (chain, combinations, combinations_with_replacement,
 from math import fsum
 
 # This module
-from iteration_utilities import PY2, PY34, _default
+from iteration_utilities import EQ_PY2, GE_PY34, _default
 # - generators
 from iteration_utilities import (accumulate, applyfunc,
                                  clamp,
@@ -32,12 +32,12 @@ from iteration_utilities import (accumulate, applyfunc,
                                  ncycles,
                                  pad, powerset,
                                  remove, repeatfunc, replace, replicate,
-                                 split, successive,
+                                 split, starfilter, successive,
                                  tabulate, tail,
                                  unique_everseen, unique_justseen)
 # - folds
 from iteration_utilities import (all_distinct, all_equal, all_monotone,
-                                 argmax, argmin,
+                                 argmax, argmin, argsorted,
                                  count_items, first, groupedby, last,
                                  minmax, nth, one, partition, second,
                                  third)
@@ -45,10 +45,12 @@ from iteration_utilities import (all_distinct, all_equal, all_monotone,
 from iteration_utilities import merge, roundrobin
 # - helper
 from iteration_utilities import all_isinstance, any_isinstance
+# - private helpers
+from iteration_utilities._cfuncs import _parse_args, _parse_kwargs
 
 
 # Conditional imports
-if PY2:
+if EQ_PY2:
     from itertools import (ifilter as filter,
                            imap as map,
                            ifilterfalse as filterfalse,
@@ -57,22 +59,19 @@ if PY2:
 else:
     from itertools import filterfalse, zip_longest
 
-if PY34:
+if GE_PY34:
     import statistics
+    from operator import length_hint
 
 
 __all__ = ['Iterable', 'InfiniteIterable', 'ManyIterables']
 
 
-def filterkwargs(**kwargs):
-    return {k: kwargs[k] for k in kwargs if kwargs[k] is not _default}
-
-
 class _Base(object):
     """Base class for method definitions that are shared by `Iterable` and
     `InfiniteIterable`.
     """
-    __slots__ = ('_iterable')
+    __slots__ = ('_iterable',)
 
     def __init__(self, iterable):
         self._iterable = iterable
@@ -104,19 +103,16 @@ class _Base(object):
     def __repr__(self):
         return '<{0.__class__.__name__}: {0._iterable!r}>'.format(self)
 
-    def _call(self, *args, **kwargs):
-        fn = args[0]
-        pos = args[1]
-        args = list(islice(args, 2, None))
-        args.insert(pos, self)
-        kwargs = filterkwargs(**kwargs)
+    def _call(self, fn, pos, *args, **kwargs):
+        args = _parse_args(args, self._iterable, pos)
+        _parse_kwargs(kwargs, _default)
         return self.__class__(fn(*args, **kwargs))
 
     def _call_finite(self, *args, **kwargs):
         res = self._call(*args, **kwargs)
         if isinstance(res, Iterable):
             return res
-        return Iterable(self._call(*args, **kwargs)._iterable)
+        return Iterable(res._iterable)
 
     def _call_infinite(self, *args, **kwargs):
         res = self._call(*args, **kwargs)
@@ -129,7 +125,7 @@ class _Base(object):
             # this Exception.
             raise TypeError('impossible to wrap an infinite iterable with '
                             'another infinite iterable.')
-        return InfiniteIterable(self._call(*args, **kwargs)._iterable)
+        return InfiniteIterable(res._iterable)
 
     @staticmethod
     def from_count(start=_default, step=_default):
@@ -150,7 +146,12 @@ class _Base(object):
         .. warning::
            This returns an `InfiniteIterable`.
         """
-        return InfiniteIterable(count(**filterkwargs(start=start, step=step)))
+        kwargs = {}
+        if start is not _default:
+            kwargs['start'] = start
+        if step is not _default:
+            kwargs['step'] = step
+        return InfiniteIterable(count(**kwargs))
 
     @staticmethod
     def from_repeat(object, times=_default):
@@ -178,8 +179,8 @@ class _Base(object):
 
     @staticmethod
     def from_itersubclasses(object):
-        """See
-        :py:func:`~iteration_utilities._recipes._additional.itersubclasses`.
+        """See \
+:py:func:`~iteration_utilities._recipes._additional.itersubclasses`.
 
         Examples
         --------
@@ -251,6 +252,9 @@ class _Base(object):
         >>> class Func:
         ...     def __init__(self):
         ...         self.val = 0
+        ...     def setlim(self, val=3):
+        ...         self.val = val
+        ...         return 'init'
         ...     def __call__(self):
         ...         self.val += 1
         ...         if self.val < 8:
@@ -259,9 +263,15 @@ class _Base(object):
 
         >>> Iterable.from_iterfunc_exception(Func(), ValueError).as_list()
         [3, 3, 3, 3, 3, 3, 3]
+
+        >>> f = Func()
+        >>> Iterable.from_iterfunc_exception(f, ValueError, f.setlim).as_list()
+        ['init', 3, 3, 3, 3]
         """
-        return Iterable(iter_except(func, exception,
-                                    **filterkwargs(first=first)))
+        if first is _default:
+            return Iterable(iter_except(func, exception))
+        else:
+            return Iterable(iter_except(func, exception, first=first))
 
     @staticmethod
     def from_repeatfunc(func, *args, **times):
@@ -292,26 +302,30 @@ class _Base(object):
 
     @staticmethod
     def from_tabulate(func, start=_default):
-        """See :py:func:`~iteration_utilities._recipes._core.tabulate`.
+        """See :py:func:`~iteration_utilities.tabulate`.
 
         Examples
         --------
-        >>> from iteration_utilities import Iterable
+        >>> from iteration_utilities import Iterable, chained
+        >>> roundint = chained(round, int)
         >>> import operator
         >>> Iterable.from_tabulate(operator.neg).islice(8).as_list()
         [0, -1, -2, -3, -4, -5, -6, -7]
 
         >>> from math import gamma
-        >>> Iterable.from_tabulate(gamma, 1).islice(8).as_tuple()
-        (1.0, 1.0, 2.0, 6.0, 24.0, 120.0, 720.0, 5040.0)
+        >>> Iterable.from_tabulate(gamma, 1).islice(8).map(roundint).as_tuple()
+        (1, 1, 2, 6, 24, 120, 720, 5040)
 
-        >>> Iterable.from_tabulate(func=gamma, start=2).islice(7).as_tuple()
-        (1.0, 2.0, 6.0, 24.0, 120.0, 720.0, 5040.0)
+        >>> Iterable.from_tabulate(func=gamma, start=2).islice(7).map(roundint).as_tuple()
+        (1, 2, 6, 24, 120, 720, 5040)
 
         .. warning::
            This returns an `InfiniteIterable`.
         """
-        return InfiniteIterable(tabulate(func, **filterkwargs(start=start)))
+        if start is _default:
+            return InfiniteIterable(tabulate(func))
+        else:
+            return InfiniteIterable(tabulate(func, start=start))
 
     def accumulate(self, func=_default, start=_default):
         """See :py:func:`~iteration_utilities.accumulate`.
@@ -331,7 +345,8 @@ class _Base(object):
         """
         return self._call(accumulate, 0, func=func, start=start)
 
-    def clamp(self, low=_default, high=_default, inclusive=_default):
+    def clamp(self, low=_default, high=_default, inclusive=_default,
+              remove=_default):
         """See :py:func:`~iteration_utilities.clamp`.
 
         Examples
@@ -342,8 +357,12 @@ class _Base(object):
 
         >>> Iterable(range(10)).clamp(low=2, high=7, inclusive=True).as_list()
         [3, 4, 5, 6]
+
+        >>> Iterable(range(10)).clamp(low=2, high=7, remove=False).as_list()
+        [2, 2, 2, 3, 4, 5, 6, 7, 7, 7]
         """
-        return self._call(clamp, 0, low=low, high=high, inclusive=inclusive)
+        return self._call(clamp, 0, low=low, high=high, inclusive=inclusive,
+                          remove=remove)
 
     def combinations(self, r):
         """See :py:func:`itertools.combinations`.
@@ -406,8 +425,8 @@ class _Base(object):
         return self._call_infinite(cycle, 0)
 
     def deepflatten(self, depth=_default, types=_default, ignore=_default):
-        """See
-        :py:func:`~iteration_utilities.deepflatten`.
+        """See \
+:py:func:`~iteration_utilities.deepflatten`.
 
         Examples
         --------
@@ -416,11 +435,11 @@ class _Base(object):
         >>> Iterable(lst).deepflatten().as_list()
         [1, 2, 3, 1, 2, 3, 1, 2, 3]
 
-        >>> Iterable(lst).deepflatten(1, (list), (str)).as_list()
+        >>> Iterable(lst).deepflatten(1, list, str).as_list()
         [1, 2, 3, 1, 2, 3, [1, 2, 3]]
 
         >>> Iterable(lst).deepflatten(depth=1,
-        ...                           types=(list), ignore=(str)).as_list()
+        ...                           types=list, ignore=str).as_list()
         [1, 2, 3, 1, 2, 3, [1, 2, 3]]
         """
         return self._call(deepflatten, 0, depth=depth, types=types,
@@ -523,8 +542,8 @@ class _Base(object):
         return self._call(flatten, 0)
 
     def getitem(self, item):
-        """"See also
-        :py:func:`~iteration_utilities._recipes._additional.getitem`
+        """See \
+:py:func:`~iteration_utilities._recipes._additional.getitem`
 
         Parameters
         ----------
@@ -583,8 +602,8 @@ class _Base(object):
             >>> it[-6::2].as_list()
             [4, 6, 8]
 
-        It's also possible to use ``get``, you have to pass in the appropriate
-        value or :py:class:`slice`::
+        It's also possible to use ``getitem`` method directly, but you have to
+        pass in the appropriate value(s) or :py:class:`slice`::
 
             >>> Iterable(range(10)).getitem(3)
             3
@@ -652,7 +671,7 @@ class _Base(object):
         return meth(islice, 0, *args)
 
     def insert(self, element, idx, unpack=_default):
-        """See also :py:func:`~iteration_utilities._recipes._additional.insert`
+        """See :py:func:`~iteration_utilities._recipes._additional.insert`
 
         Examples
         --------
@@ -695,15 +714,12 @@ class _Base(object):
         --------
         >>> from iteration_utilities import Iterable, square
         >>> Iterable(range(1, 10)).map(square).as_list()
-        {0}
+        [1, 4, 9, 16, 25, 36, 49, 64, 81]
 
         >>> Iterable(range(1, 10)).map(function=square).as_list()
-        {0}
+        [1, 4, 9, 16, 25, 36, 49, 64, 81]
         """
         return self._call(map, 1, function)
-    map.__doc__ = map.__doc__.format(
-        '[1L, 4L, 9L, 16L, 25L, 36L, 49L, 64L, 81L]' if PY2 else
-        '[1, 4, 9, 16, 25, 36, 49, 64, 81]')
 
     def ncycles(self, n):
         """See :py:func:`~iteration_utilities._recipes._core.ncycles`.
@@ -846,7 +862,9 @@ class _Base(object):
         """
         return self._call(replicate, 0, times=times)
 
-    def split(self, key, maxsplit=_default, keep=_default, eq=_default):
+    def split(self, key, maxsplit=_default,
+              keep=_default, keep_before=_default, keep_after=_default,
+              eq=_default):
         """See :py:func:`~iteration_utilities.split`.
 
         Examples
@@ -858,16 +876,35 @@ class _Base(object):
         >>> Iterable(range(1, 10)).split(is_even, 2).as_list()
         [[1], [3], [5, 6, 7, 8, 9]]
 
-        >>> Iterable(range(1, 10)).split(3, 1, True, True).as_list()
+        >>> Iterable(range(1, 10)).split(3, 1, True, False, False, True).as_list()
         [[1, 2], [3], [4, 5, 6, 7, 8, 9]]
 
+        >>> Iterable(range(1, 10)).split(3, 1, False, True, False, True).as_list()
+        [[1, 2, 3], [4, 5, 6, 7, 8, 9]]
+
+        >>> Iterable(range(1, 10)).split(3, 1, False, False, True, True).as_list()
+        [[1, 2], [3, 4, 5, 6, 7, 8, 9]]
+
         >>> Iterable(range(1, 10)).split(key=2, maxsplit=1,
         ...                              keep=True, eq=True).as_list()
         [[1], [2], [3, 4, 5, 6, 7, 8, 9]]
         """
         return self._call(split, 0, key=key, maxsplit=maxsplit, keep=keep,
+                          keep_before=keep_before, keep_after=keep_after,
                           eq=eq)
 
+    def starfilter(self, pred):
+        """See :py:func:`iteration_utilities.starfilter`.
+
+        Examples
+        --------
+        >>> from iteration_utilities import Iterable
+        >>> from operator import eq
+        >>> Iterable([1] * 20).enumerate().starfilter(eq).as_list()
+        [(1, 1)]
+        """
+        return self._call(starfilter, 1, pred)
+
     def starmap(self, function):
         """See :py:func:`itertools.starmap`.
 
@@ -973,6 +1010,186 @@ class Iterable(_Base):
     iterable : iterable
         Any kind of `iterable`.
 
+    Methods
+    -------
+    :py:meth:`~Iterable.accumulate`
+        See :py:func:`~iteration_utilities.accumulate`.
+    :py:meth:`~Iterable.as_`
+        Convert `Iterable` to other class.
+    :py:meth:`~Iterable.as_counter`
+        See :py:meth:`as_`.
+    :py:meth:`~Iterable.as_dict`
+        See :py:meth:`as_`.
+    :py:meth:`~Iterable.as_frozenset`
+        See :py:meth:`as_`.
+    :py:meth:`~Iterable.as_list`
+        See :py:meth:`as_`.
+    :py:meth:`~Iterable.as_ordereddict`
+        See :py:meth:`as_`.
+    :py:meth:`~Iterable.as_set`
+        See :py:meth:`as_`.
+    :py:meth:`~Iterable.as_string`
+        Get the iterable as string.
+    :py:meth:`~Iterable.as_tuple`
+        See :py:meth:`as_`.
+    :py:meth:`~Iterable.clamp`
+        See :py:func:`~iteration_utilities.clamp`.
+    :py:meth:`~Iterable.combinations`
+        See :py:func:`itertools.combinations`.
+    :py:meth:`~Iterable.combinations_with_replacement`
+        See :py:func:`itertools.combinations_with_replacement`.
+    :py:meth:`~Iterable.compress`
+        See :py:func:`itertools.compress`.
+    :py:meth:`~Iterable.cycle`
+        See :py:func:`itertools.cycle`.
+    :py:meth:`~Iterable.deepflatten`
+        See :py:func:`~iteration_utilities.deepflatten`.
+    :py:meth:`~Iterable.dropwhile`
+        See :py:func:`itertools.dropwhile`.
+    :py:meth:`~Iterable.duplicates`
+        See :py:func:`~iteration_utilities.duplicates`.
+    :py:meth:`~Iterable.enumerate`
+        See :py:func:`python:enumerate`.
+    :py:meth:`~Iterable.filter`
+        See :py:func:`python:filter`.
+    :py:meth:`~Iterable.filterfalse`
+        See :py:func:`itertools.filterfalse`.
+    :py:meth:`~Iterable.flatten`
+        See :py:func:`~iteration_utilities._recipes._core.flatten`.
+    :py:meth:`~Iterable.from_applyfunc`
+        See :py:func:`~iteration_utilities.applyfunc`.
+    :py:meth:`~Iterable.from_count`
+        See :py:func:`itertools.count`.
+    :py:meth:`~Iterable.from_iterfunc_exception`
+        See :py:func:`~iteration_utilities.iter_except`.
+    :py:meth:`~Iterable.from_iterfunc_sentinel`
+        See :py:func:`python:iter`.
+    :py:meth:`~Iterable.from_itersubclasses`
+        See \
+:py:func:`~iteration_utilities._recipes._additional.itersubclasses`.
+    :py:meth:`~Iterable.from_repeat`
+        See :py:func:`itertools.repeat`.
+    :py:meth:`~Iterable.from_repeatfunc`
+        See :py:func:`~iteration_utilities._recipes._core.repeatfunc`.
+    :py:meth:`~Iterable.from_tabulate`
+        See :py:func:`~iteration_utilities.tabulate`.
+    :py:meth:`~Iterable.get_all`
+        See :py:func:`python:all`.
+    :py:meth:`~Iterable.get_all_distinct`
+        See :py:func:`~iteration_utilities._cfuncs.all_distinct`.
+    :py:meth:`~Iterable.get_all_equal`
+        See :py:func:`~iteration_utilities._cfuncs.all_equal`.
+    :py:meth:`~Iterable.get_all_monotone`
+        See :py:func:`~iteration_utilities._cfuncs.all_monotone`.
+    :py:meth:`~Iterable.get_any`
+        See :py:func:`python:any`.
+    :py:meth:`~Iterable.get_argmax`
+        See :py:func:`~iteration_utilities._cfuncs.argmax`.
+    :py:meth:`~Iterable.get_argmin`
+        See :py:func:`~iteration_utilities._cfuncs.argmin`.
+    :py:meth:`~Iterable.get_argsorted`
+        See :py:func:`~iteration_utilities._recipes._additional.argsorted`.
+    :py:meth:`~Iterable.get_count_items`
+        See :py:func:`~iteration_utilities._cfuncs.count_items`.
+    :py:meth:`~Iterable.get_first`
+        See :py:func:`~iteration_utilities.nth`.
+    :py:meth:`~Iterable.get_fsum`
+        See :py:func:`math.fsum`.
+    :py:meth:`~Iterable.get_groupedby`
+        See :py:func:`~iteration_utilities._cfuncs.groupedby`.
+    :py:meth:`~Iterable.get_last`
+        See :py:func:`~iteration_utilities.nth`.
+    :py:meth:`~Iterable.get_max`
+        See :py:func:`python:max`.
+    :py:meth:`~Iterable.get_mean`
+        See :py:func:`statistics.mean`.
+    :py:meth:`~Iterable.get_median`
+        See :py:func:`statistics.median`.
+    :py:meth:`~Iterable.get_median_grouped`
+        See :py:func:`statistics.median_grouped`.
+    :py:meth:`~Iterable.get_median_high`
+        See :py:func:`statistics.median_high`.
+    :py:meth:`~Iterable.get_median_low`
+        See :py:func:`statistics.median_low`.
+    :py:meth:`~Iterable.get_min`
+        See :py:func:`python:min`.
+    :py:meth:`~Iterable.get_minmax`
+        See :py:func:`~iteration_utilities._cfuncs.minmax`.
+    :py:meth:`~Iterable.get_mode`
+        See :py:func:`statistics.mode`.
+    :py:meth:`~Iterable.get_nlargest`
+        See :py:func:`heapq.nlargest`.
+    :py:meth:`~Iterable.get_nsmallest`
+        See :py:func:`heapq.nsmallest`.
+    :py:meth:`~Iterable.get_nth`
+        See :py:func:`~iteration_utilities.nth`.
+    :py:meth:`~Iterable.get_one`
+        See :py:func:`~iteration_utilities._cfuncs.one`.
+    :py:meth:`~Iterable.get_partition`
+        See :py:func:`~iteration_utilities._cfuncs.partition`.
+    :py:meth:`~Iterable.get_pstdev`
+        See :py:func:`statistics.pstdev`.
+    :py:meth:`~Iterable.get_pvariance`
+        See :py:func:`statistics.pvariance`.
+    :py:meth:`~Iterable.get_reduce`
+        See :py:func:`functools.reduce`.
+    :py:meth:`~Iterable.get_second`
+        See :py:func:`~iteration_utilities.nth`.
+    :py:meth:`~Iterable.get_sorted`
+        See :py:func:`python:sorted`.
+    :py:meth:`~Iterable.get_stdev`
+        See :py:func:`statistics.stdev`.
+    :py:meth:`~Iterable.get_sum`
+        See :py:func:`python:sum`.
+    :py:meth:`~Iterable.get_third`
+        See :py:func:`~iteration_utilities.nth`.
+    :py:meth:`~Iterable.get_variance`
+        See :py:func:`statistics.variance`.
+    :py:meth:`~Iterable.getitem`
+        See :py:func:`~iteration_utilities._recipes._additional.getitem`
+    :py:meth:`~Iterable.grouper`
+        See :py:func:`~iteration_utilities.grouper`.
+    :py:meth:`~Iterable.insert`
+        See :py:func:`~iteration_utilities._recipes._additional.insert`
+    :py:meth:`~Iterable.intersperse`
+        See :py:func:`~iteration_utilities.intersperse`.
+    :py:meth:`~Iterable.islice`
+        See :py:func:`itertools.islice`.
+    :py:meth:`~Iterable.map`
+        See :py:func:`python:map`.
+    :py:meth:`~Iterable.ncycles`
+        See :py:func:`~iteration_utilities._recipes._core.ncycles`.
+    :py:meth:`~Iterable.pad`
+        See :py:func:`~iteration_utilities._recipes._additional.pad`.
+    :py:meth:`~Iterable.permutations`
+        See :py:func:`itertools.permutations`.
+    :py:meth:`~Iterable.powerset`
+        See :py:func:`~iteration_utilities._recipes._core.powerset`.
+    :py:meth:`~Iterable.remove`
+        See :py:func:`~iteration_utilities._recipes._additional.remove`.
+    :py:meth:`~Iterable.replace`
+        See :py:func:`~iteration_utilities._recipes._additional.replace`.
+    :py:meth:`~Iterable.replicate`
+        See :py:func:`~iteration_utilities._recipes._additional.replicate`.
+    :py:meth:`~Iterable.reversed`
+        See :py:func:`python:reversed`.
+    :py:meth:`~Iterable.split`
+        See :py:func:`~iteration_utilities.split`.
+    :py:meth:`~Iterable.starfilter`
+        See :py:func:`~iteration_utilities.starfilter`.
+    :py:meth:`~Iterable.starmap`
+        See :py:func:`itertools.starmap`.
+    :py:meth:`~Iterable.successive`
+        See :py:func:`~iteration_utilities.successive`.
+    :py:meth:`~Iterable.tail`
+        See :py:func:`~iteration_utilities._recipes._core.tail`.
+    :py:meth:`~Iterable.takewhile`
+        See :py:func:`itertools.takewhile`.
+    :py:meth:`~Iterable.unique_everseen`
+        See :py:func:`~iteration_utilities.unique_everseen`.
+    :py:meth:`~Iterable.unique_justseen`
+        See :py:func:`~iteration_utilities.unique_justseen`.
+
     Notes
     -----
 
@@ -1055,13 +1272,17 @@ class Iterable(_Base):
         [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
 
         >>> # But also some less common ones, like OrderedDict
-        >>> Iterable(range(6)).enumerate(4).as_ordered_dict()
+        >>> Iterable(range(6)).enumerate(4).as_ordereddict()
         OrderedDict([(4, 0), (5, 1), (6, 2), (7, 3), (8, 4), (9, 5)])
 
     .. warning::
        These methods are (obviously) not avaiable for `InfiniteIterable`!
     """
-    __slots__ = ('_iterable')
+    __slots__ = ('_iterable',)
+
+    if GE_PY34:
+        def __length_hint__(self):
+            return length_hint(self._iterable)
 
     def as_(self, cls):
         """Convert `Iterable` to other class.
@@ -1115,20 +1336,20 @@ class Iterable(_Base):
         {0}
         """
         return self.as_(set)
-    as_set.__doc__ = as_set.__doc__.format('set([1])' if PY2 else '{1}')
+    as_set.__doc__ = as_set.__doc__.format('set([1])' if EQ_PY2 else '{1}')
 
-    def as_frozen_set(self):
+    def as_frozenset(self):
         """See :py:meth:`as_`.
 
         Examples
         --------
         >>> from iteration_utilities import Iterable
-        >>> Iterable([5]).as_frozen_set()
+        >>> Iterable([5]).as_frozenset()
         frozenset({0})
         """
         return self.as_(frozenset)
-    as_frozen_set.__doc__ = as_frozen_set.__doc__.format('[5]' if PY2 else
-                                                         '{5}')
+    as_frozenset.__doc__ = as_frozenset.__doc__.format('[5]' if EQ_PY2 else
+                                                       '{5}')
 
     def as_dict(self):
         """See :py:meth:`as_`.
@@ -1141,13 +1362,13 @@ class Iterable(_Base):
         """
         return self.as_(dict)
 
-    def as_ordered_dict(self):
+    def as_ordereddict(self):
         """See :py:meth:`as_`.
 
         Examples
         --------
         >>> from iteration_utilities import Iterable
-        >>> Iterable(range(3, 6)).enumerate().as_ordered_dict()
+        >>> Iterable(range(3, 6)).enumerate().as_ordereddict()
         OrderedDict([(0, 3), (1, 4), (2, 5)])
         """
         return self.as_(OrderedDict)
@@ -1168,7 +1389,7 @@ class Iterable(_Base):
         """
         return self.as_(Counter)
 
-    def as_string(self, seperaror=''):
+    def as_string(self, seperator=''):
         """Get the iterable as string.
 
         .. warning::
@@ -1189,7 +1410,7 @@ class Iterable(_Base):
         >>> Iterable(range(5)).as_string(' ')
         '0 1 2 3 4'
         """
-        return seperaror.join(map(str, self._iterable))
+        return seperator.join(map(str, self._iterable))
 
     def reversed(self):
         """See :py:func:`python:reversed`.
@@ -1207,20 +1428,14 @@ class Iterable(_Base):
         """
         return self.__class__(reversed(self._iterable))
 
-    def _get(self, *args, **kwargs):
-        fn = args[0]
-        pos = args[1]
-        args = list(islice(args, 2, None))
-        args.insert(pos, self)
-        kwargs = filterkwargs(**kwargs)
+    def _get(self, fn, pos, *args, **kwargs):
+        args = _parse_args(args, self._iterable, pos)
+        _parse_kwargs(kwargs, _default)
         return fn(*args, **kwargs)
 
-    def _get_iter(self, *args, **kwargs):
-        fn = args[0]
-        pos = args[1]
-        args = list(islice(args, 2, None))
-        args.insert(pos, iter(self))
-        kwargs = filterkwargs(**kwargs)
+    def _get_iter(self, fn, pos, *args, **kwargs):
+        args = _parse_args(args, self._iterable, pos)
+        _parse_kwargs(kwargs, _default)
         return fn(*args, **kwargs)
 
     def get_all(self):
@@ -1304,19 +1519,18 @@ strict=False)
         --------
         >>> from iteration_utilities import Iterable
         >>> Iterable([1, 2, -5, 3, 4]).get_argmax()
-        4{0}
+        4
 
         >>> Iterable([1, 2, -5, 3, 4]).get_argmax(abs)
-        2{0}
+        2
 
         >>> Iterable([1, 2, -5, 3, 4]).get_argmax(key=abs)
-        2{0}
+        2
 
         >>> Iterable([]).get_argmax(key=abs, default=-1)
-        -1{0}
+        -1
         """
         return self._get(argmax, 0, key=key, default=default)
-    get_argmax.__doc__ = get_argmax.__doc__.format('L' if PY2 else '')
 
     def get_argmin(self, key=_default, default=_default):
         """See :py:func:`~iteration_utilities._cfuncs.argmin`.
@@ -1325,19 +1539,38 @@ strict=False)
         --------
         >>> from iteration_utilities import Iterable
         >>> Iterable([1, 2, -5, 3, 4]).get_argmin()
-        2{0}
+        2
 
         >>> Iterable([1, 2, -5, 3, 4]).get_argmin(abs)
-        0{0}
+        0
 
         >>> Iterable([1, 2, -5, 3, 4]).get_argmin(key=abs)
-        0{0}
+        0
 
         >>> Iterable([]).get_argmin(key=abs, default=-1)
-        -1{0}
+        -1
         """
         return self._get(argmin, 0, key=key, default=default)
-    get_argmin.__doc__ = get_argmin.__doc__.format('L' if PY2 else '')
+
+    def get_argsorted(self, key=_default, reverse=_default):
+        """See :py:func:`~iteration_utilities._recipes._additional.argsorted`.
+
+        Examples
+        --------
+        >>> from iteration_utilities import Iterable
+        >>> Iterable([1, 2, -5, 3, 4]).get_argsorted()
+        [2, 0, 1, 3, 4]
+
+        >>> Iterable([1, 2, -5, 3, 4]).get_argsorted(reverse=True)
+        [4, 3, 1, 0, 2]
+
+        >>> Iterable([1, 2, -5, 3, 4]).get_argsorted(key=abs)
+        [0, 1, 3, 4, 2]
+
+        >>> Iterable([1, 2, -5, 3, 4]).get_argsorted(abs, True)
+        [2, 4, 3, 1, 0]
+        """
+        return self._get(argsorted, 0, key=key, reverse=reverse)
 
     def get_count_items(self, pred=_default, eq=_default):
         """See :py:func:`~iteration_utilities._cfuncs.count_items`.
@@ -1346,17 +1579,15 @@ strict=False)
         --------
         >>> from iteration_utilities import Iterable
         >>> Iterable((i for i in range(10))).get_count_items()
-        10{0}
+        10
 
         >>> Iterable([1, 2, 3, 2, 1]).get_count_items(2, True)
-        2{0}
+        2
 
         >>> Iterable([1, 2, 3, 2, 1]).get_count_items(pred=2, eq=True)
-        2{0}
+        2
         """
         return self._get(count_items, 0, pred=pred, eq=eq)
-    get_count_items.__doc__ = get_count_items.__doc__.format(
-        'L' if PY2 else '')
 
     def get_first(self, default=_default, pred=_default, truthy=_default,
                   retpred=_default, retidx=_default):
@@ -1418,7 +1649,7 @@ strict=False)
                          retpred=retpred, retidx=retidx)
 
     # min and max had no default parameter before python 3.4
-    if PY2 or not PY34:
+    if not GE_PY34:
         def get_max(self, key=_default):
             """See :py:func:`max`.
 
@@ -1641,10 +1872,6 @@ strict=False)
                   retpred=_default, retidx=_default):
         """See :py:func:`~iteration_utilities.nth`.
 
-        Examples
-        --------
-        >>> from iteration_utilities import Iterable
-
         Examples
         --------
         >>> from iteration_utilities import Iterable
@@ -1659,7 +1886,7 @@ strict=False)
                          retpred=retpred, retidx=retidx)
 
     # Statistics module is only avaiable since Python 3.4
-    if PY34:
+    if GE_PY34:
         def get_mean(self):
             """See :py:func:`statistics.mean`.
 
@@ -1813,12 +2040,101 @@ class InfiniteIterable(_Base):
     corresponding methods on `Iterable` return an `InfiniteIterable` when
     appropriate. However using ``isinstance(some_iterable, InfiniteIterable)``
     could be used to determine if the :py:class:`Iterable` is infinite!
+
+    Methods
+    -------
+    :py:meth:`~InfiniteIterable.accumulate`
+        See :py:func:`~iteration_utilities.accumulate`.
+    :py:meth:`~InfiniteIterable.clamp`
+        See :py:func:`~iteration_utilities.clamp`.
+    :py:meth:`~InfiniteIterable.combinations`
+        See :py:func:`itertools.combinations`.
+    :py:meth:`~InfiniteIterable.combinations_with_replacement`
+        See :py:func:`itertools.combinations_with_replacement`.
+    :py:meth:`~InfiniteIterable.compress`
+        See :py:func:`itertools.compress`.
+    :py:meth:`~InfiniteIterable.cycle`
+        See :py:func:`itertools.cycle`.
+    :py:meth:`~InfiniteIterable.deepflatten`
+        See :py:func:`~iteration_utilities.deepflatten`.
+    :py:meth:`~InfiniteIterable.dropwhile`
+        See :py:func:`itertools.dropwhile`.
+    :py:meth:`~InfiniteIterable.duplicates`
+        See :py:func:`~iteration_utilities.duplicates`.
+    :py:meth:`~InfiniteIterable.enumerate`
+        See :py:func:`python:enumerate`.
+    :py:meth:`~InfiniteIterable.filter`
+        See :py:func:`python:filter`.
+    :py:meth:`~InfiniteIterable.filterfalse`
+        See :py:func:`itertools.filterfalse`.
+    :py:meth:`~InfiniteIterable.flatten`
+        See :py:func:`~iteration_utilities._recipes._core.flatten`.
+    :py:meth:`~InfiniteIterable.from_applyfunc`
+        See :py:func:`~iteration_utilities.applyfunc`.
+    :py:meth:`~InfiniteIterable.from_count`
+        See :py:func:`itertools.count`.
+    :py:meth:`~InfiniteIterable.from_iterfunc_exception`
+        See :py:func:`~iteration_utilities.iter_except`.
+    :py:meth:`~InfiniteIterable.from_iterfunc_sentinel`
+        See :py:func:`python:iter`.
+    :py:meth:`~InfiniteIterable.from_itersubclasses`
+        See \
+:py:func:`~iteration_utilities._recipes._additional.itersubclasses`.
+    :py:meth:`~InfiniteIterable.from_repeat`
+        See :py:func:`itertools.repeat`.
+    :py:meth:`~InfiniteIterable.from_repeatfunc`
+        See :py:func:`~iteration_utilities._recipes._core.repeatfunc`.
+    :py:meth:`~InfiniteIterable.from_tabulate`
+        See :py:func:`~iteration_utilities.tabulate`.
+    :py:meth:`~InfiniteIterable.getitem`
+        See :py:func:`~iteration_utilities._recipes._additional.getitem`
+    :py:meth:`~InfiniteIterable.grouper`
+        See :py:func:`~iteration_utilities.grouper`.
+    :py:meth:`~InfiniteIterable.insert`
+        See :py:func:`~iteration_utilities._recipes._additional.insert`
+    :py:meth:`~InfiniteIterable.intersperse`
+        See :py:func:`~iteration_utilities.intersperse`.
+    :py:meth:`~InfiniteIterable.islice`
+        See :py:func:`itertools.islice`.
+    :py:meth:`~InfiniteIterable.map`
+        See :py:func:`python:map`.
+    :py:meth:`~InfiniteIterable.ncycles`
+        See :py:func:`~iteration_utilities._recipes._core.ncycles`.
+    :py:meth:`~InfiniteIterable.pad`
+        See :py:func:`~iteration_utilities._recipes._additional.pad`.
+    :py:meth:`~InfiniteIterable.permutations`
+        See :py:func:`itertools.permutations`.
+    :py:meth:`~InfiniteIterable.powerset`
+        See :py:func:`~iteration_utilities._recipes._core.powerset`.
+    :py:meth:`~InfiniteIterable.remove`
+        See :py:func:`~iteration_utilities._recipes._additional.remove`.
+    :py:meth:`~InfiniteIterable.replace`
+        See :py:func:`~iteration_utilities._recipes._additional.replace`.
+    :py:meth:`~InfiniteIterable.replicate`
+        See :py:func:`~iteration_utilities._recipes._additional.replicate`.
+    :py:meth:`~InfiniteIterable.split`
+        See :py:func:`~iteration_utilities.split`.
+    :py:meth:`~InfiniteIterable.starfilter`
+        See :py:func:`~iteration_utilities.starfilter`.
+    :py:meth:`~InfiniteIterable.starmap`
+        See :py:func:`itertools.starmap`.
+    :py:meth:`~InfiniteIterable.successive`
+        See :py:func:`~iteration_utilities.successive`.
+    :py:meth:`~InfiniteIterable.tail`
+        See :py:func:`~iteration_utilities._recipes._core.tail`.
+    :py:meth:`~InfiniteIterable.takewhile`
+        See :py:func:`itertools.takewhile`.
+    :py:meth:`~InfiniteIterable.unique_everseen`
+        See :py:func:`~iteration_utilities.unique_everseen`.
+    :py:meth:`~InfiniteIterable.unique_justseen`
+        See :py:func:`~iteration_utilities.unique_justseen`.
+
     """
-    __slots__ = ('_iterable')
+    __slots__ = ('_iterable',)
 
 
 class ManyIterables(object):
-    __slots__ = ('_iterables')
+    __slots__ = ('_iterables',)
 
     def __init__(self, *iterables):
         """`ManyIterables` stores several `Iterables` and implements methods to
@@ -1832,6 +2148,23 @@ class ManyIterables(object):
         *iterables : any amount of iterables
             The `iterables` to store.
 
+        Methods
+        -------
+        :py:meth:`~ManyIterables.chain`
+            See :py:func:`itertools.chain`.
+        :py:meth:`~ManyIterables.map`
+            See :py:func:`python:map`.
+        :py:meth:`~ManyIterables.merge`
+            See :py:func:`~iteration_utilities.merge`.
+        :py:meth:`~ManyIterables.product`
+            See :py:func:`itertools.product`.
+        :py:meth:`~ManyIterables.roundrobin`
+            See :py:func:`~iteration_utilities.roundrobin`.
+        :py:meth:`~ManyIterables.zip`
+            See :py:func:`python:zip`.
+        :py:meth:`~ManyIterables.zip_longest`
+            See :py:func:`itertools.zip_longest`.
+
         Notes
         -----
         This is just a convenience class to seperate the expressions dealing
@@ -1869,8 +2202,8 @@ class ManyIterables(object):
         """
         self._iterables = iterables
 
-    __init__.__doc__ = __init__.__doc__.format('itertools.imap' if PY2
-                                               else 'map')
+    __init__.__doc__ = __init__.__doc__.format(
+        'itertools.imap' if EQ_PY2 else 'map')
 
     def _call(self, fn, infinitecheck, *args, **kwargs):
         iterables = self._iterables
@@ -1882,7 +2215,8 @@ class ManyIterables(object):
             cls = Iterable
         if args:
             iterables = args + iterables
-        return cls(fn(*iterables, **filterkwargs(**kwargs)))
+        _parse_kwargs(kwargs, _default)
+        return cls(fn(*iterables, **kwargs))
 
     def chain(self):
         """See :py:func:`itertools.chain`.
